const { getStreamFromURL } = global.utils;
const tinyurl = require('tinyurl');
const axios = require("axios");

const API = 'https://api3.akashxzh.workers.dev';

module.exports = {
  config: {
    name: "aniwatch",
    version: "1.0",
    author: "Akash",
    countDown: 20,
    role: 0,
    longDescription: "Search and get info",
    category: "video",
    guide: { en: "Usage: {pn} name." },
  },

    onStart: async function ({ args, event, message }) {
      try {
        
        const query = args.join(' ');
       if (query.length === 0) {
         return message.reply("Please provide a anime name for search.")
       }
        const response = await search(query);
        const results = response.results;
        let Ids = [];
        let combinedData = "";
        let stream = [];

        for (let index = 0; index < Math.min(results.length, 6); index++) {
          const item = results[index];
          combinedData += `${index + 1}. ${item.title}\n${item.releaseDate}\n\n`;
          Ids.push(item.id);
          stream.push(await getStreamFromURL(item.img));
        }

        const extra = results.slice(6);

        await message.reply({
          body: combinedData + "Reply with 'next' to get the next page results",
          attachment: stream,
        }, async (err, info) => {
          if (err) {
            console.error(err);
            message.reply(`Error: ${err}`);
            return;
          }

          global.GoatBot.onReply.set(info.messageID, {
            commandName: this.config.name,
            messageID: info.messageID,
            author: event.senderID,
            type: "page",
            ids: Ids,
            remainingResults: extra
          });
        });

      } catch (error) {
        console.error(error);
        message.reply(`Error: ${error.message}`);
      }
    },

    onReply: async function ({ event, Reply, message }) {
      try {
        const { author, ids, remainingResults, type, epIds, infoImg, totalEp } = Reply;
        const { senderID, body } = event;

        if (senderID !== author) {
          return;
        }

        if (type === "page") {
          if (body.toLowerCase().includes('next')) {
            if (!remainingResults.length > 0) {
              return message.reply("No more pages available.");
            }
            let newIds = [];
            let combinedData = "";
            let stream = [];

            for (let index = 0; index < Math.min(remainingResults.length, 6); index++) {
              const item = remainingResults[index];
              combinedData += `${index + 1}. ${item.title}\n${item.releaseDate}\n\n`;
              newIds.push(item.id);
              stream.push(await getStreamFromURL(item.img));
            }

            const extra = remainingResults.slice(6);

            await message.reply({
              body: combinedData + "Reply with 'next' to get the next page results",
              attachment: stream,
            }, async (err, info) => {
              if (err) {
                console.error(err);
                message.reply(`Error: ${err}`);
                return;
              }

              global.GoatBot.onReply.set(info.messageID, {
           commandName: this.config.name,
              messageID: info.messageID,
                author: event.senderID,
                type: "page",
                ids: newIds,
                remainingResults: extra
              });
            });
          } else if (body.toLowerCase().includes('info')) {
           
          const number = parseInt(body.split(' ')[1]);
        if (isNaN(number) || number < 1 || number > ids.length) {
       return message.reply("Invalid info number!");
         } 

            const infoId = ids[number - 1];
            const response = await getInfo(infoId);
            const infos = response.results;
            const episodeIds = infos.episodes.map(episode => episode[1]);
            
            message.reply({
              body: `${infos.name}\n\nStatus: ${infos.status}\nReleased: ${infos.released}\nType: ${infos.type}\nGenres: ${infos.genre}\nTotal Episode: ${infos.episodes.length}\n\n${infos.plot_summary}`,
              attachment: await getStreamFromURL(infos.image)
            }, async (err, info) => {
              if (err) {
                console.error(err);
                message.reply(`Error: ${err}`);
                return;
              }
              global.GoatBot.onReply.set(info.messageID, {
                commandName: this.config.name,
                messageID: info.messageID,
                author: event.senderID,
                type: "download",
                epIds: episodeIds,
                infoImg: infos.image,
                totalEp: infos.episodes.length
              });
            });
          }
        } else if (type === "download") {
          const number = parseInt(body);
          if (isNaN(number) || number < 1 || number > totalEp) {
            return message.reply("Please enter a valid episode number!");
          }
          const data = await watch(epIds[number - 1]);
         const DL = await download(epIds[number - 1]);
const link = DL.results;
const HD1080p = await tinyurl.shorten(link["1920x1080"]);
const SD = await tinyurl.shorten(link["640x360"]);
const streamingLink = await tinyurl.shorten(data.results.stream.sources[0].file);

message.reply({
  body: `Title: ${data.results.name}\n\nDownload Links:\nHD : ${HD1080p}\nSD : ${SD}\nStreaming Link:\nHD : ${streamingLink}`,
  attachment: await getStreamFromURL(infoImg)
});
        }
      } catch (error) {
        message.reply(`Error: ${error}`);
      }
    }
};

async function search(query) {
  const url = API + '/search/' + query;
  const response = await axios.get(url);
  return response.data;
};

async function getInfo(id) {
  const url = API + '/anime/' + id;
  const response = await axios.get(url);
  return response.data;
};

async function watch(id) {
  const url = API + '/episode/' + id; 
  const response = await axios.get(url);
  return response.data;
}

async function download(id) {
  const url = API + '/download/' + id; 
  const response = await axios.get(url);
  return response.data;
}