const axios = require('axios');

module.exports = {
	config: {
		name: "autotrans",
		aliases: ["at"],
		version: "1.1",
		author: "Loufi | Allou Mohamed",
		countDown: 5,
		role: 0,
		shortDescription: {
			en: "Translate text"
		},
		longDescription: {
			en: "Translate into any language ✅"
		},
		category: "Services",
		guide: {
			en: "Reply to a person:\n{pn} [on | off] language"
		}
	},

	onStart: async function ({ message, event, args, usersData }) {
		let UID;
		let langset;
		let onoff;
		if (args[0]?.match(/^\d+$/)) {
			UID = args[1];
			langset = args[2] || 'ar';
			onoff = args[0];
		} else if (Object.keys(event.mentions || {}).length) {
			UID = Object.keys(event.mentions)[0];
			langset = args[1] || 'ar';
			onoff = args[0];
		} else if (event.messageReply?.senderID) {
			UID = event.messageReply.senderID;
			langset = args[1] || 'ar';
			onoff = args[0];
		} else {
			UID = event.senderID;
			langset = args[1] || 'ar';
			onoff = args[0];
			}
		if (!UID || !langset || !onoff) return message.reply('🤧');
		if (langset.length !== 2) return message.reply('set valid lang code');
		await usersData.set(UID, onoff, "settings.translate");
		await usersData.set(UID, langset, "settings.translateautolang");
	},

	onChat: async function ({ event, usersData, message }) {
		const senderID = event.senderID;
		const langset = await usersData.get(senderID, "settings.translateautolang");
		const translateEnabled = await usersData.get(senderID, "settings.translate");
		
		if (event.body && translateEnabled === "on") {
			const translatedMessage = await translateAndSendMessage(event.body, langset);
			message.reply(translatedMessage);
		}
	}
};

async function translate(text, langCode) {
	const res = await axios.get(`https://translate.googleapis.com/translate_a/single?client=gtx&sl=auto&tl=${langCode}&dt=t&q=${encodeURIComponent(text)}`);
	return {
		text: res.data[0].map(item => item[0]).join(''),
		lang: res.data[2]
	};
}

async function translateAndSendMessage(content, langCodeTrans) {
	const { text, lang } = await translate(content, langCodeTrans);
	return `${text}\n\nTranslation to ${langCodeTrans} (${lang})`;
}