const path = require("path");
const fs = require("fs");
const axios = require("axios");
const sharp = require("sharp");

function generateRandomWord(length) {
  var word = "";
  var characters = "abcdefghijklmnopqrstuvwxyz0123456789";
  var charactersLength = characters.length;
  for (var i = 0; i < length; i++) {
    word += characters.charAt(Math.floor(Math.random() * charactersLength));
  }
  return word;
}

const rw = generateRandomWord(5);

const imageX = path.join(__dirname, "cache", `combined-${rw}.jpg`);

module.exports = {
  config: {
    name: "gen2",
    version: "1.0",
    author: "Akash Xzh",
    countDown: 5,
    role: 0,
    shortDescription: "",
    longDescription: "",
    category: "img",
    guide: {
      en: ".",
    },
  },

  onStart: async function ({ api, args, event, message }) {
    try {
      setTimeout(() => {
        message.reaction("⏳", event.messageID);
      }, 700);

      const text = args.join(" ");

      if (!text) {
        return message.reply("Please provide a prompt.");
      }

      let prompt, ratio;
      if (text.includes("|")) {
        const [promptText, ratioText] = text.split("|").map((str) => str.trim());
        prompt = promptText;
        ratio = ratioText;
      } else {
        prompt = text;
        ratio = "1";
      }

      const imagePath = [];
      const imagePromises = [];
      const url = `https://dalle3.autodox.repl.co/create?prompt=${prompt}`;
      const headers = {
        "User-Agent": "Testing Group",
      };

      const response = await axios.get(url, { headers });
      const data = response.data.image_urls;
      const time = response.data.time_taken;
      if (!data) {
        return message.reply(response.data.error);
      }
      for (const url of data) {
        imagePath.push(url);
      }

      const imageUrls = await Promise.all(imagePath);

      const gridImage = await createGridImage(imageUrls, ratio);
      const gridImagePath = imageX;

      const attachments = [fs.createReadStream(gridImagePath)];

      message.reply(
        {
          body: `✅ | Image successfully generated.\n\nPlease respond with a number\nFrom 1 to 4 to receive the\nRespective High-resolution image.\n\nTime taken: ${time}`,
          attachment: attachments,
        },
        async (err, info) => {
          let id = info.messageID;
          global.GoatBot.onReply.set(info.messageID, {
            commandName: this.config.name,
            messageID: info.messageID,
            author: event.senderID,
            imageUrls: imageUrls,
          });
        }
      );  
    } catch (error) {
      console.error(error);
      message.reply(`Error: ${error}`);
    }
  },

  onReply: async function ({ api, event, Reply, usersData, args, message }) {
    const reply = parseInt(args[0]);
    const { author, messageID, imageUrls } = Reply;

    if (event.senderID !== author) return;
    try {
      if (reply >= 1 && reply <= 4) {
        const imglink = imageUrls[reply - 1];

        message.reply({
          attachment: await global.utils.getStreamFromURL(imglink),
        });
      } else {
        message.reply("Invalid image number. Please reply with a number between 1 to 4.");
      }
    } catch (error) {
      console.error(error);
      message.reply(`Error: ${error}`);
    } 
  },
};

async function createGridImage(imageUrls, ratio) {
  let width;
  let height;
  if (ratio === "1") {
    width = 1024;
    height = 1024;
  } else if (ratio === "2") {
    width = 1792;
    height = 2688;
  } else if (ratio === "3") {
    width = 3360;
    height = 1640;
  } else {
    throw new Error("Invalid ratio");
  }

  const images = imageUrls.map((url) => axios.get(url, { responseType: "arraybuffer" }));

  const buffers = await Promise.all(images).then((responses) =>
    responses.map((response) => Buffer.from(response.data))
  );

  const resizedBuffers = await Promise.all(
    buffers.map((buffer) => sharp(buffer).resize(width / 2, height / 2).toBuffer())
  );

  const gridImageBuffer = await sharp({
    create: {
      width,
      height,
      channels: 4,
      background: { r: 255, g: 255, b: 255, alpha: 1 },
    },
  })
    .composite([
      { input: resizedBuffers[0], top: 0, left: 0 },
      { input: resizedBuffers[1], top: 0, left: width / 2 },
      { input: resizedBuffers[2], top: height / 2, left: 0 },
      { input: resizedBuffers[3], top: height / 2, left: width / 2 },
    ])
    .png()
    .toBuffer();

  fs.writeFileSync(imageX, gridImageBuffer);

  return gridImageBuffer;
}