const Encly = require('encly');

module.exports = {
  config: {
    name: "genshin",
    aliases: ["charinfo", "ginfo"],
    version: "1.0",
    author: "Akash Xzh",
    cooldown: 20,
    role: 2,
    longDescription: "Fetch genshin build cards.",
    category: "tools",
    guide: { en: "Usage: {pn} uid" },
  },

  onStart: async function ({ args, event, api, message }) {
  const uid = args[0];
  if (!uid) {
    return message.reply("Please provide a UID. 🤔");
  }
  message.reply("Processing your request. Please wait... ⏳");

  try {
    const card = new Encly.Card();
    let cards = await card.getCard(parseInt(uid));

    if (cards.length === 0) {
      return message.reply(`No cards found for the provided UID. Try again... ❌ ${cards}`);
    }

    const data = new Encly.Profile();
    const profileInfo = await data.getProfile(parseInt(uid));

    let replyMessage = `Profile Information:\n👤 Name: ${profileInfo.nickname}\n🔢 Level: ${profileInfo.level}\n🏆 Achievements: ${profileInfo.achievements}\n🌀 Abyss: ${profileInfo.abyssFloor}-${profileInfo.abyssLevel}\n\n`;

    cards.forEach((item, index) => {
      replyMessage += `${index + 1}. ${item.name}\n`;
    });

    replyMessage += "\nTo view build cards, reply with the corresponding number.";

    message.reply(replyMessage, (err, info) => {
      if (err) {
        console.error(err);
        api.sendMessage(`Error: ${err}`, event.threadID);
        return;
      }

      global.GoatBot.onReply.set(info.messageID, {
        commandName: this.config.name,
        messageID: info.messageID,
        author: event.senderID,
        nick: profileInfo.nickname,
        id: uid,
        names: cards.map(card => card.name),
        urls: cards.map(card => card.url)
      });
    });
  } catch (error) {
    if (error instanceof AggregateError) {
      for (const err of error) {
        console.error(err);
        message.reply("An error occurred while processing your request. ❗");
      }
    } else {
      console.error(error);
      message.reply(error.message || "An error occurred while processing your request. ❗");
    }
  }
},

  onReply: async function ({ api, event, Reply, message }) {
    let { author, messageID, urls, nick, id, names } = Reply;
    if (event.senderID !== author) return;

    let option = parseInt(event.body);
    if (isNaN(option) || option < 1 || option > urls.length) {
      return message.reply("Invalid option. Please choose a number within the range. ❌");
    }

    const input = urls[option - 1];
    message.reply("Loading... ⏳");
    message.reply({
      body: `📜 Here's your requested build card\n\n👤 Name: ${nick}\n🎭 Character: ${names[option -1]}\n🆔 UID: ${id}`,
      attachment: await global.utils.getStreamFromURL(input)
    });
  }
};