const { Prodia } = require("prodia.js");
const fs = require("fs");
const axios = require("axios");

const prodia = new Prodia("d0896ca7-caca-49bf-b115-3f2b5c5872d5");

const config = {
  name: "imagine2",
  aliases: ["gen2"],
  version: "1.0",
  author: "JARiF@একটু পাত্তা দওনা চেটে চেটে খাব",
  countDown: 10,
  role: 0,
  shortDescription: {
    vi: "",
    en: "text 2 image",
  },
  longDescription: {
    vi: "",
    en: "make your imagination",
  },
  category: "Ai",
  guide: {
    vi: "",
    en: "",
  },
};

async function imageGenerator(model, prompt, filePath) {
  let job = await prodia.createJob({
    model: model,
    prompt: prompt,
    negative_prompt: "deformed, bad anatomy, disfigured, poorly drawn face, mutation, badly drawn, mutated, extra limb, ugly, disgusting, poorly drawn hands, missing limb, floating limbs, disconnected limbs, malformed hands, blurry, ((((mutated hands and fingers)))), watermark, watermarked, oversaturated, censored, distorted hands, amputation, missing hands, obese, doubled face, double hands, asian, b&w, black and white, sepia",
    steps: 50,
    cfg_scale: 7,
    seed: -1,
    upscale: false,
    sampler: "DPM++ 2M Karras",
    aspect_ratio: "square",
  });

  while (job.status !== "succeeded") {
    await new Promise((resolve) => setTimeout(resolve, 250));
    job = await prodia.getJob(job.job);
  }

  const imageUrl = job.imageUrl;

  const imageStream = await axios.get(imageUrl, { responseType: 'stream' });
  const fileStream = fs.createWriteStream(filePath);

  imageStream.data.pipe(fileStream);

  return new Promise((resolve, reject) => {
    fileStream.on('finish', () => resolve(filePath));
    fileStream.on('error', reject);
  });
}

module.exports = {
  config,

  onStart: async function ({ message, event, args, api }) {
    const prompt = args.join(" ");

    if (prompt.length <= 2) {
      return message.reply("The prompt is too short...");
    }

    message.reply("✅| Creating your Imagination...", (err, info) => { id = info.messageID });

    const filePaths = ["output1.png", "output2.png", "output3.png", "output4.png"];
    const models = ["Realistic_Vision_V2.0.safetensors [79587710]","revAnimated_v122.safetensors [3f4fefd9]", "dreamshaper_6BakedVae.safetensors [114c8abb]", "anything-v4.5-pruned.ckpt [65745d25]","openjourney_V4.ckpt [ca2f377f]"]; 

    const selectedModels = [];
    const selectedImages = [];

    // Generate images using different models
    for (let i = 0; i < 4; i++) {
      const modelIndex = Math.floor(Math.random() * models.length);
      const selectedModel = models[modelIndex];
      selectedModels.push(selectedModel);

      const filePath = filePaths[i];
      const imagePromise = imageGenerator(selectedModel, prompt, filePath);
      selectedImages.push(imagePromise);
    }

    Promise.all(selectedImages)
      .then((outputFilePaths) => {
        console.log("Images saved:", outputFilePaths);
        const collageFilePath = "collage.png";
        // Combine the images into a collage
        // You can use any library or algorithm of your choice to create the collage
        // Here's a simple example using the `canvas` package
        const Canvas = require("canvas");
        const canvas = Canvas.createCanvas(800, 800);
        const ctx = canvas.getContext("2d");

        const imagePromises = outputFilePaths.map((filePath, index) => {
          return new Promise((resolve, reject) => {
            const img = new Canvas.Image();
            img.onload = () => {
              const x = (index % 2) * 400;
              const y = Math.floor(index / 2) * 400;
              ctx.drawImage(img, x, y, 400, 400);
              resolve();
            };
            img.onerror = reject;
            img.src = filePath;
          });
        });

        Promise.all(imagePromises)
          .then(() => {
            // Save the collage image
            const out = fs.createWriteStream(collageFilePath);
            const stream = canvas.createPNGStream();
            stream.pipe(out);
            out.on("finish", () => {
              // Send the collage image in the message
              message.reply({ attachment: fs.createReadStream(collageFilePath) }, event.threadID);

              // Display the prompt for selecting the image number
              message.reply("Here's your imagination 🖼️. Please reply with the image number (1, 2, 3, 4) to get the corresponding image in high resolution.")
                .then((msgInfo) => {
                  const collector = api.messageCollector.create({
                    event: 'message',
                    userID: event.senderID,
                    attachedInfo: msgInfo,
                    callback: (collectorMessage) => {
                      const imageNumber = parseInt(collectorMessage.body);
                      if (imageNumber >= 1 && imageNumber <= 4) {
                        const selectedImagePath = outputFilePaths[imageNumber - 1];
                        // Send the high-resolution image
                        message.reply({ attachment: fs.createReadStream(selectedImagePath) }, event.threadID);
                      } else {
                        message.reply("Invalid image number. Please reply with a number between 1 and 4.");
                      }
                      // Stop the message collector
                      collector.stop();
                    }
                  });
                })
                .catch((error) => {
                  console.error("Error displaying prompt:", error);
                  return message.reply("");
                });
            });
          })
          .catch((error) => {
            console.error("Error creating collage:", error);
            return message.reply("An error occurred while generating the collage.");
          });
      })
      .catch((error) => {
        console.error("Error generating images:", error);
        return message.reply("An error occurred while generating the images.");
      });
  },
};