const fs = require("fs");
const path = require("path");

module.exports = {
	config: {
		name: "help2",
		version: "1.0",
		author: "JARiF",
		countDown: 20,
		role: 0,
		category: "info",
	},

	onStart: async function ({ message, args, event, threadsData, getLang, role, api }) {
		const chatId = message.chat.id;
		const commandName = args.length ? args.join(' ') : null;

		if (!commandName) {
			const categories = {};
			const uncategorized = [];

			const scriptFiles = fs.readdirSync(__dirname).filter(file => file.endsWith('.js') && file !== 'config.js');

			scriptFiles.forEach(file => {
				const scriptPath = path.join(__dirname, file);
				try {
					const { config } = require(scriptPath);
					if (config && config.category) {
						if (!categories[config.category]) {
							categories[config.category] = [];
						}
						categories[config.category].push(file.replace('.js', ''));
					} else {
						uncategorized.push(file.replace('.js', ''));
					}
				} catch (error) {
					console.error(`Error loading config for file: ${file}`, error);
				}
			});

			let configMessage = '';
			let totalCommands = 0;

			for (const category in categories) {
				configMessage += `╭──『 ${category.toUpperCase()} 』\n`;
				let commandCount = 0;
				for (const command of categories[category]) {
					if (commandCount === 3) {
						configMessage += '\n';
						commandCount = 0;
					}
					configMessage += `✧${command} `;
					commandCount++;
				}
				configMessage += '\n╰───────────◊\n';
				totalCommands += categories[category].length;
			}

			if (uncategorized.length > 0) {
				configMessage += `╭──『 UNCATEGORIZED 』\n`;
				let commandCount = 0;
				for (const command of uncategorized) {
					if (commandCount === 3) {
						configMessage += '\n';
						commandCount = 0;
					}
					configMessage += `✧${command} `;
					commandCount++;
				}
				configMessage += '\n╰───────────◊\n';
				totalCommands += uncategorized.length;
			}

			configMessage += '╭────────────◊\n';
			configMessage += `│ » Total commands: ${totalCommands}\n`;
			configMessage += '│ » A Powerful Telegram bot\n';
			configMessage += '│ » By Eijah & JARiF\n';
			configMessage += '╰────────◊\n';
			configMessage += '「 Annie Bot 」';

			message.reply(configMessage);
		} else {
			const scriptPath = path.join(__dirname, `${commandName}.js`);
			if (fs.existsSync(scriptPath)) {
				try {
					const { config } = require(scriptPath);
					if (config && typeof config === 'object') {
						const { name, version, author, countDown, role, description, category } = config;
						const infoMessage = `Command: ${name}\n` +
							`❏ Version: ${version}\n` +
							`❏ Author: ${author}\n` +
							`❏ Cooldown: ${countDown}s\n` +
							`❏ Can use: ${role}\n` +
							`❏ Description: ${description}\n` +
							`❏ Category: ${category || 'Uncategorized'}`;

						message.reply(infoMessage);
					} else {
						message.reply(`No config available for ${commandName}.`);
					}
				} catch (error) {
					console.error(`Error loading config for command: ${commandName}`, error);
					message.reply(`Error loading config for ${commandName}.`);
				}
			} else {
				message.reply(`Command ${commandName} not found.`);
			}
		}
	}
};