const axios = require("axios");

module.exports = {
    config: {
        name: "logo",
        countDown: 10,
        author: "Akash Xzh",
        category: "ai imagination",
        role: 0,
        guide: {
            en: "Usage: {pn} prompt\nExample: {pnt} cute girl --ar 2:3\n\nSupported ratios: 1:1, 2:3, 3:2"
        }
    },
    onStart: async function ({ args, event, message }) {
        try {
            setTimeout(() => {
                message.reaction("⏳", event.messageID);
            }, 700);

            const text = args.join(" ");
            if (!text) {
                return message.reply("Please provide a prompt.");
            }

            message.reply("✅ | Processing your image......");

            const prompt = text.includes("--ar") ? text.split("--ar ")[0] : text;
            let ratio;

            switch (true) {
                case text.includes("--ar 1:1"):
                    ratio = 1;
                    break;
                case text.includes("--ar 2:3"):
                    ratio = 2;
                    break;
                case text.includes("--ar 3:2"):
                    ratio = 3;
                    break;
                default:
                    ratio = 1;
                    break;
            }

            const response = await axios.post(
                'https://api.replicate.com/v1/predictions',
                {
                    'version': '58a7099052ed9928ee6a65559caa790bfa8909841261ef588686660189eb9dc8',
                    'input': {
                        'prompt': prompt,
                        'negative_prompt': 'lowres, bad anatomy, bad hands, error, missing fingers, cropped, worst',
                        'width': getWidth(ratio),
                        'height': getHeight(ratio),
                        'steps': 30,
                        'upscale': false,
                        'upscale_steps': '0'
                    }
                },
                {
                    headers: {
                        'Content-Type': 'application/json',
                        'Authorization': 'Token r8_80lNb8AwpEemw9n39hGmIaUKvwF86WG3ruxIG'
                    },
                    validateStatus: status => true
                }
            );

            const getUrl = response.data.urls.get;
            let output = null;

            while (!output) {
                const poll = await axios.get(getUrl, {
                    headers: {
                        'Authorization': 'Token r8_80lNb8AwpEemw9n39hGmIaUKvwF86WG3ruxIG'
                    }
                });

                output = poll.data.output;
            }

            let streams = [];

            for (let i of output) {
                const s = await global.utils.getStreamFromURL(i);
                streams.push(s);
            }

            if (streams.length > 0) {
                await message.reply({
                    body: "Image generated.",
                    attachment: streams,
                });
            } else {
                throw new Error("Failed to retrieve generated image.");
            }

            message.reaction("✅", event.messageID);
        } catch (error) {
            console.error(error);
            await message.reply(`Error occurred during image generation.`);
        }
    }
};

function getWidth(ratio) {
    const widths = [1024, 1024, 1024];
    return widths[ratio - 1] || 1024;
}

function getHeight(ratio) {
    const heights = [1024, 640, 960];
    return heights[ratio - 1] || 1024;
}