const axios = require('axios');
const cheerio = require('cheerio');
const { getStreamFromURL } = global.utils;
const link = require('tinyurl');

module.exports = {
  config: {
    name: "news",
    version: "1.1",
    author: "Duck",
    shortDescription: {
      en: "Gets you the latest headlines from Prothom Alo"
    },
    longDescription: {
      en: "Gets you the latest headlines from Prothom Alo"
    },
    category: "news",
    countDown: 10,
    role: 0,
    guide: {
      en: "{pn}"
    }
  },

  onStart: async function ({ message, event, args }) {
    try {
      const categories = {
        latest: { slug: 'latest', limit: 5 },
        cyber: { slug: 'cyberworld-science-tech-education', limit: 20 },
        crime: { slug: 'crime-bangladesh', limit: 10 },
        entertainment: { slug: 'fun', limit: 15 },
        politics: { slug: 'politics', limit: 10 },
        sports: { slug: 'sports-photo', limit: 15 },
        football: { slug: 'football-sports', limit: 10 }, cricket: { slug: 'cricket-sports', limit: 10 },
        health: { slug: 'health-life-style', limit: 10 },
        economy: { slug: 'bank-economy', limit: 10 },
        science: { slug: 'science-education', limit: 10 },
        technology: { slug: 'technology-photo', limit: 10 },
      };

      let slug, limit;

      if (args[0] in categories) {
        slug = categories[args[0]].slug;
        limit = categories[args[0]].limit;
      } else {
        const categoryList = Object.keys(categories).map((key, index) => `${index + 1}. ${key}`).join('\n');
        return message.reply(`Choose a category by responding with the number:\n${categoryList}`, (err, info) => {
          global.GoatBot.onReply.set(info.messageID, {
            commandName: this.config.name,
            author: event.senderID,
            messageID: info.messageID,
            type: 'category',
            slugs: Object.keys(categories),
            limits: Object.values(categories),
          });
        });
      }
    } catch (error) {
      console.error(error);
    }
  },

  onReply: async function ({ api, event, Reply, usersData, args, message, type }) {
    try {
      const reply = parseInt(args[0]);
      const { author, messageID, slugs, limits, type } = Reply;
      if (event.senderID != author) return;

      if (type == 'category') {
        try {
          const selectedCategory = limits[reply - 1];
          api.unsendMessage(messageID) && api.setMessageReaction("⏰", event.messageID, () => {}, true);
          if (selectedCategory) {
            const { slug, limit } = selectedCategory;

            const response = await axios.get('https://www.prothomalo.com/v1/mobile/collection', {
              params: {
                slug: slug,
                limit: limit,
                offset: 0,
                'story-fields': 'headline,created-at,published-at,updated-at,subheadline,author-name,summary,url,hero-image-s3-key'
              },
              headers: {
                'Accept': 'application/json',
                'Connection': 'Keep-Alive',
                'User-Agent': 'okhttp/3.12.0'
              }
            });

            const items = response.data.items;
            const newsDetails = [];
            const images = [];

            for (let i = 0; i < items.length; i++) {
              const item = items[i];

            //  const heroImg = await global.utils.getStreamFromUrl(`https://images.prothomalo.com/${item['hero-image-s3-key']}`);
           //   images.push(heroImg);

              newsDetails.push(`╭─────────────⭓\n├───⭓ 📰 News ${i + 1}\n- 🌟 Headline: ${item.headline}\n- 📝 Summary: ${item.summary}\n- 📅 Date: ${new Date(item['published-at']).toLocaleString('bn-BD', { timeZone: 'Etc/GMT-6' })}\n- 🔗 Link: ${await link.shorten(item.url)}\n╰─────────────⭓\n\n`);
            }

            const bodyText = newsDetails.join('') + "Reply with the news number to view details.";
    api.setMessageReaction("✅", event.messageID, () => {}, true);
            await message.reply({ body: bodyText }, async (err, info) => {
              let id = info.messageID;
              global.GoatBot.onReply.set(id, {
                commandName: this.config.name,
                messageID: id,
                author: event.senderID,
                type: 'details',
                headlines: items.map(item => item.headline),
                urls: items.map(item => item.url),
              });
            });
          }
        } catch (error) {
          console.error(error);
          message.reply(`Error: ${error}`);
        }
      } else if (type == 'details') {
        const { urls, headlines } = Reply;

        try {
          if (reply >= 1 && reply <= urls.length) {
            const urlData = urls[reply - 1];
const title = headlines[reply - 1];

api.setMessageReaction("⏰", event.messageID, () => {}, true);

async function fetchWebpage(url) {
  try {
    const response = await axios.get(url);
    return response.data;
  } catch (error) {
    console.error('Error fetching webpage:', error.message);
    throw error;
  }
}

function extractParagraphs(htmlContent) {
  const $ = cheerio.load(htmlContent);
  const paragraphs = $('p').map((index, element) => $(element).text()).get();
  return paragraphs;
}

async function main() {
  try {
    const url = urlData;
    const htmlContent = await fetchWebpage(url);
    const paragraphs = extractParagraphs(htmlContent);
    let content = "";
    paragraphs.forEach((paragraph, index) => {
      content += `${paragraph.trim()}\n\n`;
    });
    message.reply(`Headline: ${title}\n\n${content}`);
  } catch (error) {
    console.error('Error in main function:', error.message);
  }
}

main();


           await api.setMessageReaction("✅", event.messageID, () => {}, true);
          } else {
            message.reply("Invalid number. Please reply with a number between 1 to " + urls.length);
          }
        } catch (error) {
          console.error(error);
          message.reply(`Error: ${error}`);
        }
      }
    } catch (error) {
      console.error(error);
      message.reply(`Error: ${error}`);
    }
  }
};