const { getStreamsFromAttachment } = global.utils
const fs = require('fs-extra');

module.exports = {
  config: {
    name: "notiall",
    version: "1.1",
    author: "Akash Xzh",
    role: 0,
    category: "owner",
  
    guide: {
      en: "{pn} text",
    },
    envConfig: {
      delayPerGroup: 250
    }
  },
  langs: {
    en: {
      missingInput: "Please enter the message you want to send to all groups",
      notification: "Notification from admin bot to all users (do not reply to this message)",
      sendingNoti: "Start sending notification from admin bot to all bot users"
      
    }
  },
  onStart: async function ({ message, api, event, args, envCommands, threadsData, usersData, getLang, commandName }) {
    const { delayPerGroup } = envCommands[commandName];
    if (!args[0]) {
      return message.reply(getLang("missingInput"));
    }
    message.reply(getLang("sendingNoti"));
    const formSend = {
      body: `${getLang("notification")}\n────────────────\n${args.join(" ")}`,
      attachment: await getStreamsFromAttachment(
        [
          ...event.attachments,
          ...(event.messageReply?.attachments || [])
        ].filter(item => ["photo", "png", "animated_image", "video", "audio"].includes(item.type))
      )
    };
    
    const allUserData = await usersData.getAll();
    const allUserIDs = allUserData.map((user) => user.userID);
    const userCount = allUserIDs.length;

    const allThreadIDs = (await threadsData.getAll())
      .filter(t => t.isGroup && t.members.some(m => m.userID === api.getCurrentUserID()))
      .map(t => t.threadID);
    const threadCount = allThreadIDs.length;

    async function sendMessageWithDelay(userID, threadID) {
      try {
        await api.sendMessage(formSend, userID);
        await sleep(delayPerGroup);
        await api.sendMessage(formSend, threadID);
        await sleep(delayPerGroup);
      } catch (e) {
        console.log(e);
      }
    }

    async function sleep(ms) {
      return new Promise((resolve) => setTimeout(resolve, ms));
    }

    for (const [i, userID] of allUserIDs.entries()) {
      const threadID = allThreadIDs[i];
      await sendMessageWithDelay(userID, threadID);
    }

    message.reply(`✅ Sent notification to ${userCount} users and ${threadCount} groups successfully`);
  },
};