const axios = require("axios");


const llm = "llama-3-sonar-large-32k-online";
const apiUrl = 'https://simoai-llmapi.onrender.com';

module.exports = {
  config: {
    name: "sonar",
    aliases: ["llama3"],
    version: "1.0",
    author: "SiAM",
    countDown: 10,
    role: 0,
    shortDescription: {
      vi: "",
      en: ""
    },
    longDescription: {
      vi: "",
      en: "Latest llm"
    },
    category: "GPT",
    guide: {
      en: "{pn} 'query'\nexample:\n{pn} hi there"
    }
  },
  onStart: async function ({ api, message, event, args, commandName }) {
    const userID = event.senderID;
    let query = encodeURIComponent(args.join(" "));
    if (!query) {
      message.reply("Please provide a query. \n\nExample: /sonar hey there ");
      return;
    }



    try {
      message.reaction("⏰", event.messageID);
      const response = await axios.get(`${apiUrl}/perplexity/chat?prompt=${query}&id=${userID}&model=${llm}`);

      console.log(response.data);

      const answer = response.data.answer;
      
      await message.reaction("✅", event.messageID);

      if (answer) {
        message.reply(
          {
            body: answer,
            attachment: ''
          },

          (err, info) => {
            global.GoatBot.onReply.set(info.messageID, {
              commandName,
              messageID: info.messageID,
              author: event.senderID
            });
          }
        );
      } else {
        console.error("Invalid API Response:", response.data);
        sendErrorMessage(message, "Server response is invalid ❌");
      }
    } catch (error) {
      console.error("Request Error:", error);
      sendErrorMessage(message, "Server not responding ❌");
    }
  },
  onReply: async function ({ message, event, Reply, args }) {
    let { author, commandName } = Reply;
    if (event.senderID !== author) return;

    if (args[0].toLowerCase() === "clear") {
      try {
        const userID = author;
        const response = await axios.get(`${apiUrl}/perplexity/chat?prompt=clear&id=${userID}&model=${llm}`);


      } catch (error) {
        console.error("Request Error:", error);
        sendErrorMessage(message, "Server not responding ❌");
      }
      global.GoatBot.onReply.delete(message.messageID);

    }

    const userID = author;
    const query = encodeURIComponent(args.join(" "));

    try {
      message.reaction("⏰", event.messageID);
      const response = await axios.get(`${apiUrl}/perplexity/chat?prompt=${query}&id=${userID}&model=${llm}`);

      const answer = response.data.answer;
      
     message.reaction("✅", event.messageID);

      if (answer) {
        message.reply(
          {
            body: answer,
            attachment: ''
          },

          (err, info) => {
            global.GoatBot.onReply.set(info.messageID, {
              commandName,
              messageID: info.messageID,
              author: event.senderID
            });
          }
        );
      } else {
        console.error("Invalid API Response:", response.data);
        sendErrorMessage(message, "Server response is invalid ❌");
      }
    } catch (error) {
      console.error("Request Error:", error);
      sendErrorMessage(message, "Server not responding ❌");
    }
    global.GoatBot.onReply.delete(message.messageID);
  }
};

function sendErrorMessage(message, errorMessage) {
  message.reply({ body: errorMessage });
}