const fs = require('fs');
const util = require('util');
const request = util.promisify(require('request'));
const tiny = require('tinyurl');
const runAsync = util.promisify(require('child_process').execFile);
const runSync = util.promisify(require('child_process').execFileSync);

module.exports = {
    config: {
        name: "spdl",
        version: "1.0",
        author: "Duck",
        countDown: 0,
        role: 0,
        category: "Music",
    },
    onStart: async function ({ message, args, event }) {
        try {
            message.reply("Processing your request...");

            if (event.type === "message_reply") {
                const attachmentUrl = tiny.shorten(event.messageReply.attachments[0].url);
                const data = {
                    'url': attachmentUrl,
                    'return': 'apple_music,spotify',
                    'api_token': '15c49d5f40b6b39fba3519fa6cb332c7'
                };
                const response = await request({
                    uri: 'https://api.audd.io/',
                    form: data,
                    method: 'POST'
                });

                const apiResult = JSON.parse(response.body).result;

                if (!apiResult) {
                    return message.reply("No result found!");
                }

                const spotifyUrl = apiResult.spotify.external_urls.spotify;
                await processMusic(spotifyUrl);
            } else {
                const query = args.join(' ');
                await processMusic(query);
            }
        } catch (error) {
            return message.reply(`ERROR: ${error.message}`);
        }

        async function processMusic(query) {
            try {
                if (!query) {
                    return message.reply("Missing input!");
                }

                const fileName = Math.floor(Date.now() / 1000);
                const filePath = `spotdl/${fileName}.opus`;
                const metadataFilePath = `spotdl/${fileName}.spotdl`;

                const spotdl = runSync('spotdl', ['download', `"${query}"`, '--threads', '8', '--bitrate', 'disable', '--format', 'opus', '--output',
                    filePath, '--save-file', metadataFilePath]);
                
                spotdl.on('error', (err) => {
                    return message.reply(`ERROR: ${err}`)
                });
                
                spotdl.stderr.on('data', (data) => {
                    return message.reply(`ERROR: ${data}`)
                });
                
                spotdl.on('close', (code) => {
                    if (code == 0) {
                        const metadata = JSON.parse(fs.readFileSync(metadataFilePath, 'utf8'));
                        message.reply({
                            body: `• Title: ${metadata.name}\n• Artist: ${metadata.artist}\n• Album: ${metadata.album_name}\n• Date: ${metadata.date}\n• Publisher: ${metadata.publisher}\n• Spotify URL: ${metadata.url}`,
                            attachment: fs.createReadStream(filePath)
                        });
                    }
                });
                
                await runAsync('rm', ['-r', filePath]);
                await runAsync('rm', ['-r', metadataFilePath]);

            } catch (error) {
                throw new Error(`ERROR: ${error.message}`);
            }
        }
    }
};