const fs = require('fs');

module.exports = {
  config: {
    name: "wordgame",
    version: "1.2",
    author: "Ncs Pro",
    countDown: 30,
    role: 0,
    shortDescription: {
      vi: "",
      en: "Guess the scrambled word within 30 seconds to earn money!"
    },
    longDescription: {
      vi: "",
      en: "A fun game where you have to unscramble a word within 30 seconds to win a cash reward."
    },
    category: "game",
    guide: {
      en: "{pn}"
    },
    envConfig: {
      reward: 25
    }
  },

  langs: {
    en: {
      reply: "Here's the scrambled word. You have only 30 seconds to guess it:\n",
      correct: "Congratulations, you got the right answer and won %1 dollars!",
      wrong: "Oops, wrong answer. Try again!"
    }
  },

  onStart: async function ({ message, event, commandName, getLang, args }) {
    const json = JSON.parse(fs.readFileSync('./scramble.json', 'utf8'));

    const Qdata = json[Math.floor(Math.random() * json.length)];

    if (!Qdata || !Qdata.question) {
      return;
    }

    const text = Qdata.question.split('\n')[0];
    const choices = Qdata.question.split('\n').slice(1).join('\n');
    const body = Qdata.answer;

    message.reply({
      body: `${getLang('reply')}\n\n${text}\n\n${choices}`
    }, (err, info) => {
      global.GoatBot.onReply.set(info.messageID, {
        commandName,
        messageID: info.messageID,
        author: event.senderID,
        answer: Qdata.answer,
        answered: false,
      });

      setTimeout(() => {
        if (!global.GoatBot.onReply.has(info.messageID)) {
          return;
        }
        message.unsend(info.messageID);
        global.GoatBot.onReply.delete(info.messageID);
      }, this.config.countDown * 1000);
    });
  },

  onReply: async ({ message, Reply, event,getLang, usersData, envCommands, commandName }) => {
    const { author, messageID, answer, answered } = Reply;

    if (answered || author != event.senderID) {
      return;
    }

    if (formatText(event.body) == formatText(answer)) {
      global.GoatBot.onReply.delete(messageID);
      message.reply(getLang("correct", envCommands[commandName].reward));
      await usersData.addMoney(event.senderID, envCommands[commandName].reward);
    }
    else {
      global.GoatBot.onReply.set(messageID, { ...Reply, answered: true });
      message.reply(getLang("wrong"));
    }

    message.unsend(event.messageReply.messageID);
  }
};

function formatText(text) {
  return text.normalize("NFD")
             .toLowerCase()
}