const { getStreamFromURL } = global.utils;

const axios = require('axios');


module.exports = {

  config: {

    name: "suno",

    version: "1.0",

    author: "Metimol",

    cooldown: 30,

    role: 0,

    longDescription: "Text to Music",

    category: "tools",

    guide: { en: "{pn} prompt" },

  },


  onStart: async function ({ args, event, message }) {

    try {

      let prompt;

      if (args[0] === "create") {

        prompt = args.slice(1).join(' ');

      } else {

        prompt = args.join(' ');

      }

      if (!prompt) {

        return message.reply('Missing Prompt!');

      }

      const w = await message.reply("Generating Music, Please wait...");

      let gen;

      if (args[0] === "create") {

       let promptX, title, tag;

       let text = args.slice(1).join(' ');

       if (text.includes("|")) {

         const [promptT, titles, tags] = text.split("|").map((str) => str.trim());

         promptX = promptT;

         title = titles;

         tag = tags;

       } else {

         promptX = text;

         title = "UNTITLED";

         tag = "pop electronic";

       }

        gen = await customRequest(promptX, title, tag);

      } else {

        gen = await request(prompt);

      }

      console.log(gen);

      let body;

      if (gen.audio_url.length === 2) {

         body = `Music generated\nTitle: ${gen.title}\nReact accordingly to get your desired result:\n\nVideo 1: 😆\nVideo 2: 😠\nAudio 1: 😢\nAudio 2: 😮`;

      } else {

        body = `Music generated\nTitle: ${gen.title}\nReact accordingly to get your desired result:\n\nVideo 1: 😢\nAudio 1: 😠`;

      }

      let streams = [];

      for (let i of gen.image_url) {

        const s = await getStreamFromURL(i);

        streams.push(s);

      }

      message.reply({

        body: body,

        attachment: streams,

      }, async (err, info) => {

        if (err) {

          console.error(err);

          message.reply(`Error: ${err.message}`);

          return;

        }

        message.unsend(w.messageID);

        global.GoatBot.onReaction.set(info.messageID, {

          commandName: this.config.name,

          messageID: info.messageID,

          author: event.senderID,

          lyric: gen.lyrics,

          video: gen.video_url,

          audio: gen.audio_url

        });

      });

    } catch (error) {

      console.error(error);

      message.reply(`Error: ${error.message}`);

    }

  },


  onReaction: async function ({ event, Reaction, message }) {

    try {

      const { author, lyric, video, audio } = Reaction;

      const { userID } = event;

      if (userID !== author)

        return;

      const reaction = event.reaction;

      const r1 = [ '😮', '😠', '😆', '😢' ];


         if (!r1.includes(reaction)) {

          return;

        }

       let url = number(reaction, audio, video);

       message.reply("⏰");

   message.reply({

        body: lyric,

        attachment: await getStreamFromURL(url)

      });

    } catch (error) {

      console.error(error);

      message.reply(`Error: ${error.message || "Unknown error"}`);

    }

  }

};


function number(reaction, audio, video) {

  switch (reaction) {

    case '😆':

      return video[0];

    case '😠':

      return video[1];

    case '😢':

      return audio[0];

    case '😮':

      return audio[1];

    default:

      return null;

  }

}


async function request(prompt) {

  try {

    const generate = await axios.post('https://osan-katywilliams1121s-projects.vercel.app/api/generate', {

      prompt: prompt,

      make_instrumental: false,

      wait_audio: false

    }, {

      headers: {

        'accept': 'application/json',

        'Content-Type': 'application/json'

      }

    });

    const resources = await result(generate.data);

    return resources;

  } catch (error) {

    throw new Error('Error fetching data:\n' + error);

  }

}


async function customRequest(promptX, title, tag) {

  try {

    const generate = await axios.post('https://osan-katywilliams1121s-projects.vercel.app/api/custom_generate', {

      prompt: promptX, 

      tags: tag,

      title: title,

      make_instrumental: false,

      wait_audio: false

    }, {

      headers: {

        'Accept': 'application/json',

        'Content-Type': 'application/json'

      }

    });

    const resources = await result(generate.data);

    return resources;

  } catch (error) {

    throw new Error('Error fetching data:\n' + error);

  }

}


async function result(generatedData) {

  try {

    const ids = generatedData.map(item => item.id).join(',');

    while (true) {

      const response = await axios.get(`https://osan-katywilliams1121s-projects.vercel.app/api/get?ids=${ids}`, {

        headers: {

          'accept': 'application/json'

        }

      });

      if (response.data && response.data.length === 2 &&

          response.data[0].status === 'complete' && response.data[1].status === 'complete') {

        return {

          title: response.data[0].title,

          lyrics: `${response.data[0].lyric}, ${response.data[1].lyric}`,

          audio_url: [response.data[0].audio_url, response.data[1].audio_url],

          image_url: [response.data[0].image_url, response.data[1].image_url],

          video_url: [response.data[0].video_url, response.data[1].video_url]

        };

      } else if (response.data && response.data.length === 1 && response.data[0].status === 'complete') {

        return {

          title: response.data[0].title,

          lyrics: response.data[0].lyric,

          audio_url: response.data[0].audio_url,

          image_url: response.data[0].image_url,

          video_url: response.data[0].video_url

        };

      }

      await new Promise(resolve => setTimeout(resolve, 20000));

    }

  } catch (error) {

    throw new Error('Error fetching data:\n' + error);

  }

}