const axios = require("axios");
const { getStreamFromURL } = global.utils;

module.exports = {
	config: {
		name: "t3i",
		aliases: ["midjourney"],
		version: "1.1",
		author: "JARiF@একটু পাত্তা দওনা চেটে চেটে খাব",
		countDown: 30,
		role: 0,
		shortDescription: {
			vi: "Tạo ảnh từ văn bản của bạn",
			en: "Create image from your text"
		},
		longDescription: {
			uid: "Tạo ảnh từ văn bản của bạn",
			en: "Create image from your text"
		},
		category: "Ai",
		guide: {
			vi: "   {pn} <prompt>: tạo ảnh từ văn bản của bạn",
			en: "   {pn} <prompt>: create image from your text"
		}
	},

	langs: {
		vi: {
			syntaxError: "⚠️ Vui lòng nhập prompt",
			error: "❗ Đã có lỗi xảy ra, vui lòng thử lại sau"
		},
		en: {
			syntaxError: "⚠️ Please enter prompt",
			error: "❗ An error has occurred, please try again later"
		}
	},

	onStart: async function ({ message, args, getLang }) {
  try {
    const prompt = args.join(" ");
    if (!prompt) {
      return message.reply(getLang("syntaxError"));
    }

    await message.reply("Processing your image,dear.....");

    const data = await midJourney(prompt, {});
    const imageUrl = data[0];
    const imageStream = await getStreamFromURL(imageUrl, "openjourney.png");

    return message.reply({
      attachment: imageStream
    });
  } catch (error) {
    console.error(error);
    return message.reply("Something went wrong.");
  }
  } 
}

const ReplicateUtils = {
	run: async function (model, inputs) {
		let prediction;
		try {
			prediction = await this.create(model, inputs);
		}
		catch (e) {
			throw e.response.data;
		}
		while (![
			'canceled',
			'succeeded',
			'failed'
		].includes(prediction.status)) {
			await new Promise(_ => setTimeout(_, 250));
			prediction = await this.get(prediction);
		}

		return prediction.output;
	},

	async get(prediction) {
		if (prediction.prediction)
			return prediction.prediction;
		const controller = new AbortController();
		const id = setTimeout(() => controller.abort(), 29000);
		const response = await axios.get(`https://replicate.com/api/models${prediction.version.model.absolute_url}/versions/${prediction.version_id}/predictions/${prediction.uuid}`, {
			signal: controller.signal
		}).then(r => r.data);
		clearTimeout(id);
		return response;
	},

	create(model, inputs) {
		const [path, version] = model.split(':');

		return axios({
			url: `https://replicate.com/api/models/${path}/versions/${version}/predictions`,
			method: 'POST',
			headers: {
				'Content-Type': 'application/json'
			},
			data: JSON.stringify({ inputs })
		})
			.then(response => response.data);
	}
};

const model = "stability-ai/stable-diffusion:db21e45d3f7023abc2a46ee38a23973f6dce16bb082a930b0c49861f96d1e5bf";
const midJourney = async (prompt, parameters = {}) => await ReplicateUtils.run(model, { prompt, ...parameters });