const axios = require('axios');


module.exports = {

    config: {

        name: "anim",

        version: "1.1",

        author: "Akash Xzh",

        shortDescription: { en: "Not Available" },

        longDescription: { en: "Not Available" },

        category: "ai imagination",

        countDown: 10,

        role: 0,

        guide: { en: "{pn} prompt" }

    },

    onStart: async function ({ event, args, message }) {

        try {

            const text = args.join(' ');

            if (!text) {

                return message.reply("Provide a prompt.");

            }

            const { ratio, lora, prompt, model } = filterCnv(text);

            message.reply("Creating...");

            const requestId = randomId();

            const { width, height } = ratioCnv(ratio);

            const response = await axios.post('https://ap-east-1.tensorart.cloud/v1/jobs', {

                "requestId": requestId,

                "stages": [

                    {

                        "type": "INPUT_INITIALIZE",

                        "inputInitialize": {

                            "seed": "-1",

                            "count": 1

                        }

                    },

                    {

                        "type": "DIFFUSION",

                        "diffusion": {

                            "width": width,

                            "height": height,

                            "prompts": [{ "text": prompt }],

                            "negativePrompts": [{ "text": "lowres, (bad), error, fewer, extra, missing, worst quality, jpeg artifacts, low quality, unfinished, displeasing, oldest, early, chromatic aberration, signature, extra digits, artistic error, scan, abstract" }],

                            "sdModel": model,

                            "sdVae": "Automatic",

                            "sampler": "Euler a",

                            "steps": 30,

                            "cfgScale": 5,

                            "clipSkip": 0,

                            "etaNoiseSeedDelta": 0,

                            "lora": {

                                "items": [{ "loraModel": lora, "weight": 1 }]

                            }

                        }

                    }

                ],

                "engine": "TAMS_V2"

            }, {

                headers: {

                    'Content-Type': 'application/json',

                    'Authorization': 'Bearer fd6d4f466cdf453b97426c7ba910f132'

                }

            });

            const jobId = response.data.job.id;

            const jobDetails = await getJobDetails(jobId);

            if (jobDetails.job.successInfo.images.length > 0) {

                message.reply({ body: "Here's your image.", attachment: await global.utils.getStreamFromURL(jobDetails.job.successInfo.images[0].url) });

            } else {

                message.reply("No image found.");

            }

        } catch (error) {

            console.error("Error occurred:", error);

            message.reply(error.message);

        }

    }

};


function filterCnv(text) {

    const ratioFlagIndex = text.indexOf("--ar");

    const loraFlagIndex = text.indexOf("--lora");

    const modelFlagIndex = text.indexOf("--model");

    let prompt = text;

    let ratio = "1:1";

    let lora = "719797520777336105";

    let model = "706485991197825616";


    if (ratioFlagIndex !== -1) {

        const ratioStartIndex = ratioFlagIndex + 5;

        const ratioEndIndex = text.indexOf(" ", ratioStartIndex);

        const ratioText = ratioEndIndex !== -1 ? text.substring(ratioStartIndex, ratioEndIndex).trim() : text.substring(ratioStartIndex).trim();

        const ratioRegex = /(\d+):(\d+)/;

        const ratioMatch = ratioText.match(ratioRegex);


        if (ratioMatch) {

            ratio = ratioMatch[1] + ':' + ratioMatch[2];

            prompt = text.replace(/--ar\s+\S+/, '').trim();

        }

    }


    if (loraFlagIndex !== -1) {

        const loraStartIndex = loraFlagIndex + 6;

        const loraEndIndex = text.indexOf("--", loraStartIndex);

        if (loraEndIndex !== -1) {

            lora = text.substring(loraStartIndex, loraEndIndex).trim();

            prompt = text.substring(0, loraFlagIndex).trim() + " " + text.substring(loraEndIndex).trim();

        } else {

            lora = text.substring(loraStartIndex).trim();

            prompt = text.substring(0, loraFlagIndex).trim();

        }

    }


    const modelRegex = /--model\s+(\d+)/;

    const modelMatch = text.match(modelRegex);


    if (modelMatch) {

        model = modelMatch[1];

        prompt = text.replace(/--model\s+\S+/, '').trim();

    }


    prompt = prompt.replace(/--\w+\s+\S+/g, '').trim();

    return { ratio, lora, model, prompt };

}


function ratioCnv(ratio) {

    const [widthRatio, heightRatio] = ratio.split(":").map(Number);

    const maxWidth = 2048;

    const width = maxWidth / (widthRatio + heightRatio) * widthRatio;

    const height = width / widthRatio * heightRatio;

    return { width: Math.round(width), height: Math.round(height) };

}


function randomId() {

    return Math.random().toString(36).substring(2, 15) + Math.random().toString(36).substring(2, 15);

}


async function getJobDetails(jobId) {

    try {

        while (true) {

            const response = await axios.get(`https://ap-east-1.tensorart.cloud/v1/jobs/${jobId}`, {

                headers: {

                    'Authorization': 'Bearer fd6d4f466cdf453b97426c7ba910f132'

                }

            });

            if (response.data.job.status === "SUCCESS") {

                return response.data;

            }

            await new Promise(resolve => setTimeout(resolve, 10000));

        }

    } catch (error) {

        console.error("Error getting job details:", error);

        throw error;

    }

}
