const axios = require('axios');

module.exports = {
  config: {
    name: "ethereal",
    version: "1.1",
    author: "Akash Xzh",
    shortDescription: {
      en: "Not Available"
    },
    longDescription: {
      en: "Not Available"
    },
    category: "Memes",
    countDown: 10,
    role: 0,
    guide: {
      en: "{pn} prompt"
    }
  },

  onStart: async function ({ event, args, message }) {
    try {
      let prompt;
      let aspectRatio = "512:640";
      let argsExtra = [];

      for (let i = 0; i < args.length; i++) {
        if (args[i].startsWith("--ar")) {
          aspectRatio = args[i].split("=")[1];
        } else {
          if (args[i].startsWith("--")) {
            argsExtra.push(args[i]);
          } else {
            prompt = args.slice(i).join(" ");
            break;
          }
        }
      }

      if (!prompt) {
        return message.reply("Please provide a prompt.");
      }

      message.reply("Creating...");

      const requestId = randomId();

      const { width, height } = ratioCnv(aspectRatio);

      const response = await axios.post('https://ap-east-1.tensorart.cloud/v1/jobs', {
        "requestId": requestId,
        "stages": [
          {
            "type": "INPUT_INITIALIZE",
            "inputInitialize": {
              "seed": "-1",
              "count": 1
            }
          },
          {
            "type": "DIFFUSION",
            "diffusion": {
              "width": width,
              "height": height,
              "prompts": [
                {
                  "text": prompt
                }
              ],
              "negativePrompts": [
                {
                  "text": "verybadimagenegative_v1.3, (worst quality, low quality:1.4), (malformed hands:1.4), (mutated fingers:1.4), text, signature, simple background"
                }
              ],
              "sdModel": "670899266572730064",
              "sdVae": "kl-f8-anime.ckpt",
              "sampler": "DPM++ 2M Karras",
              "steps": 30,
              "cfgScale": 7,
              "clipSkip": 1,
              "etaNoiseSeedDelta": 0
            }
          },
          {
            "type": "IMAGE_TO_UPSCALER",
            "image_to_upscaler": {
              "hr_upscaler": "4x-AnimeSharp",
              "hr_scale": 4,
              "hr_second_pass_steps": 20,
              "denoising_strength": 0.39
            }
          }
        ],
        "engine": "TAMS_V2"
      }, {
        headers: {
          'Content-Type': 'application/json',
          'Authorization': 'Bearer bf24fae785474f3b8fa3b80bf7153b1d'
        }
      });

      const jobId = response.data.job.id;

      const jobDetails = await getJobDetails(jobId);
      if (jobDetails.job.successInfo.images.length > 0) {
        message.reply({
          body: "Here's your image.",
          attachment: await global.utils.getStreamFromURL(jobDetails.job.successInfo.images[0].url)
        });
      } else {
        message.reply("No image found.");
      }

    } catch (error) {
      console.error("Error occurred:", error);
      message.reply(error.message);
    }
  }
};

function ratioCnv(ratio) {
  const [widthRatio, heightRatio] = ratio.split(":").map(Number);
  const maxWidth = 1024;
  const width = maxWidth / (widthRatio + heightRatio) * widthRatio;
  const height = width / widthRatio * heightRatio;
  return { width: Math.round(width), height: Math.round(height) };
}

function randomId() {
  return Math.random().toString(36).substring(2, 15) + Math.random().toString(36).substring(2, 15);
}

async function getJobDetails(jobId) {
  try {
    while (true) {
      const response = await axios.get(`https://ap-east-1.tensorart.cloud/v1/jobs/${jobId}`, {
        headers: {
          'Authorization': 'Bearer bf24fae785474f3b8fa3b80bf7153b1d'
        }
      });

      if (response.data.job.status === "SUCCESS") {
        return response.data;
      }

      await new Promise(resolve => setTimeout(resolve, 10000));
    }
  } catch (error) {
    console.error("Error getting job details:", error);
    throw error;
  }
}