const axios = require('axios');

module.exports = {
  config: {
    name: "pastel",
    version: "1.1",
    author: "Akash Xzh",
    shortDescription: {
      en: "Not Available"
    },
    longDescription: {
      en: "Not Available"
    },
    category: "Memes",
    countDown: 10,
    role: 0,
    guide: {
      en: "{pn} prompt"
    }
  },

  onStart: async function ({ event, args, message }) {
    try {
      const prompt = args.join(" ");
      if (!prompt) {
        return message.reply("Please provide a prompt.");
      }
      message.reply("Creating...");

      const requestId = randomId();

      const response = await axios.post('https://ap-east-1.tensorart.cloud/v1/jobs', {
        "requestId": requestId,
        "stages": [
          {
            "type": "INPUT_INITIALIZE",
            "inputInitialize": {
              "seed": "-1",
              "count": 1
            }
          },
          {
            "type": "DIFFUSION",
            "diffusion": {
              "width": 512,
              "height": 512,
              "prompts": [
                {
                  "text": prompt
                }
              ],
              "negativePrompts": [
                {
                  "text": "bad-hands-5, bad-artist-anime"
                }
              ],
              "sdModel": "682133908881593549",
              "sdVae": "kl-f8-anime2.ckpt",
				"sampler": "DPM++ 2M SDE Karras",
				"steps": 30,
				"cfgScale": 7,
				"clipSkip": 2,
				"etaNoiseSeedDelta": 0,
     "denoising_strength": 0.55
            }
          },
          {
            "type": "IMAGE_TO_UPSCALER",
            "image_to_upscaler": {
              "hr_upscaler": "Latent (nearest)",
              "hr_scale": 1.5,
              "hr_second_pass_steps": 25
            }
          }
        ],
        "engine": "TAMS_V2"
      }, {
        headers: {
          'Content-Type': 'application/json',
          'Authorization': 'Bearer bf24fae785474f3b8fa3b80bf7153b1d'
        }
      });

      const jobId = response.data.job.id;

      const jobDetails = await JobDetails(jobId);
      if (jobDetails.job.successInfo.images.length > 0) {
        message.reply({
          body: "Here's your image.",
          attachment: await global.utils.getStreamFromURL(jobDetails.job.successInfo.images[0].url)
        });
      } else {
        message.reply("No image found.");
      }

    } catch (error) {
      console.error("Error occurred:", error);
      message.reply(error.message);
    }
  }
};

function randomId() {
  return Math.random().toString(36).substring(2, 15) + Math.random().toString(36).substring(2, 15);
}

async function JobDetails(jobId) {
  try {
    while (true) {
      const response = await axios.get(`https://ap-east-1.tensorart.cloud/v1/jobs/${jobId}`, {
        headers: {
          'Authorization': 'Bearer bf24fae785474f3b8fa3b80bf7153b1d'
        }
      });

      if (response.data.job.status === "SUCCESS") {
        return response.data;
      }

      await new Promise(resolve => setTimeout(resolve, 10000));
    }
  } catch (error) {
    console.error("Error getting job details:", error);
    throw error;
  }
}