const axios = require('axios');

module.exports = {
  config: {
    name: "rizz",
    version: "1.1",
    author: "Akash Xzh",
    shortDescription: {
      en: "Create images based on prompts."
    },
    longDescription: {
      en: "This command generates images based on the provided prompt."
    },
    category: "Memes",
    countDown: 10,
    role: 0,
    guide: {
      en: "{pn} prompt"
    }
  },

  onStart: async function ({ event, args, message }) {
    try {
      const prompt = args.join(" ");
      if (!prompt) {
        return message.reply("Please provide a prompt.");
      }
      message.reply("Creating...");

      const requestId = randomId();

      const response = await axios.post('https://ap-east-1.tensorart.cloud/v1/jobs', {
        "requestId": requestId,
        "stages": [
          {
            "type": "INPUT_INITIALIZE",
            "inputInitialize": {
              "seed": "-1",
              "count": 1
            }
          },
          {
            "type": "DIFFUSION",
            "diffusion": {
              "width": 768,
              "height": 1152,
              "prompts": [{ "text": prompt }],
              "negativePrompts": [{ "text": "lowres, bad anatomy, bad hands" }],
              "sdModel": "659716967030304328",
              "sampler": "Euler a",
              "steps": 45,
              "cfgScale": 7,
              "clipSkip": 5,
              "etaNoiseSeedDelta": 31337,
              "v1Clip": false,
              "enableHr": true,
              "hrUpscaler": "Latent",
              "hrSecondPassSteps": 20,
              "denoisingStrength": 0.5,
              "hrResizeX": 768,
              "hrResizeY": 1152,
              "adetailer": {
                "enableAdetailer": true,
                "args": [{
                  "adModel": "face_yolov8n.pt",
                  "adConfidence": 0.3,
                  "adMaskBlur": 4,
                  "adDenoisingStrength": 0.4,
                  "adInpaintOnlyMasked": true
                }]
              }
            }
          }
        ],
        "engine": "TAMS_V1"
      }, {
        headers: {
          'Content-Type': 'application/json',
          'Authorization': 'Bearer bf24fae785474f3b8fa3b80bf7153b1d'
        }
      });

      const jobId = response.data.job.id;

      const jobDetails = await getJobDetails(jobId);
      if (jobDetails.job.successInfo.images.length > 0) {
        message.reply({
          body: "Here's your image.",
          attachment: await getImageStream(jobDetails.job.successInfo.images[0].url)
        });
      } else {
        message.reply("No image found.");
      }

    } catch (error) {
      console.error("Error occurred:", error);
      message.reply(error.message);
    }
  }
};

function randomId() {
  return Math.random().toString(36).substring(2, 15) + Math.random().toString(36).substring(2, 15);
}

async function getJobDetails(jobId) {
  try {
    while (true) {
      const response = await axios.get(`https://ap-east-1.tensorart.cloud/v1/jobs/${jobId}`, {
        headers: {
          'Authorization': 'Bearer bf24fae785474f3b8fa3b80bf7153b1d'
        }
      });

      if (response.data.job.status === "SUCCESS") {
        return response.data;
      }

      await new Promise(resolve => setTimeout(resolve, 10000));
    }
  } catch (error) {
    console.error("Error getting job details:", error);
    throw error;
  }
}

async function getImageStream(url) {
  const response = await axios.get(url, { responseType: 'stream' });
  return response.data;
}