const axios = require('axios');
const cheerio = require('cheerio');
const fs = require('fs');

module.exports = {
  config: {
    name: "videoig",
    version: "1.3",
    author: "YourName",
    countDown: 5,
    role: 0,
    shortDescription: {
      vi: "Tải video từ Instagram",
      en: "Download video from Instagram"
    },
    longDescription: {
      vi: "Tải video từ Instagram",
      en: "Download video from Instagram"
    },
    category: "media",
    guide: {
      en: "{pn} <video-url>: Download video from Instagram"
    }
  },

  langs: {
    vi: {
      missingUrl: "Vui lòng nhập URL video Instagram bạn muốn tải về.",
      error: "Đã xảy ra lỗi khi tải video.",
      downloading: "Đang tiến hành tải video cho bạn.",
      tooLarge: "Rất tiếc, không thể tải video vì dung lượng lớn hơn 83MB."
    },
    en: {
      missingUrl: "Please enter the Instagram video URL you want to download.",
      error: "An error occurred while downloading the video.",
      downloading: "Downloading video for you.",
      tooLarge: "Sorry, we can't download the video because the size is larger than 83MB."
    }
  },

  onStart: async function ({ args, message, getLang }) {
    if (!args[0]) {
      return message.reply(getLang("missingUrl"));
    }

    const url = args[0];
    const response = await igDownloader(url);

    if (response.success === false) {
      return message.reply(getLang("error"));
    }

    let success = false;
    const msgSend = message.reply(getLang("downloading"));

    for (const item of response.download) {
      try {
        const res = await axios({
          url: item.url,
          responseType: 'stream' 
        });

        const filePath = `./tmp/${global.utils.randomString(10)}.mp4`;
        const writer = fs.createWriteStream(filePath); 

        await new Promise((resolve, reject) => {
          res.data.pipe(writer); 

          writer.on('finish', resolve);
          writer.on('error', reject);
        });

        const stats = fs.statSync(filePath);
        const fileSizeInBytes = stats.size;
        const fileSizeInMB = fileSizeInBytes / (1024 * 1024);

        if (fileSizeInMB > 83) {
          fs.unlinkSync(filePath); 
          continue;
        }

        const fileName = `${global.utils.randomString(10)}.mp4`;
        const attachment = {
          name: fileName,
          file: fs.readFileSync(filePath) 
        };

        message.reply({
          attachment
        }, async () => message.unsend((await msgSend).messageID));

        success = true;
        fs.unlinkSync(filePath); 
        break;
      } catch (err) {
        console.error(err);
      }
    }

    if (!success) {
      message.unsend((await msgSend).messageID);
      return message.reply(getLang("tooLarge"));
    }
  }
};

async function igDownloader(url) {
  try {
    const response = await axios.get(`https://www.saveinsta.com/download?url=${encodeURIComponent(url)}`);
    const $ = cheerio.load(response.data);
    const videoUrl = $('.download_box > .row > .col-sm-6:nth-child(2) > a').attr('href');
    const download = [];

    if (videoUrl) {
      download.push({
        quality: 'Unknown',
        url: videoUrl
      });
    }

    return {
      success: true,
      download
    };
  } catch (err) {
    console.error(err);
    return {
      success: false
    };
  }
}
