const os = require('os');
const { execSync } = require('child_process');

function formatUptime(uptime, SystemUptime) {
  const days = Math.floor(uptime / (60 * 60 * 24));
  const hours = Math.floor((uptime % (60 * 60 * 24)) / (60 * 60));
  const minutes = Math.floor((uptime % (60 * 60)) / 60);
  const seconds = Math.floor(uptime % 60);

SystemUptimeForm = SystemUptime.replace(/^up\s+/i, '');

  return `🤖 Bot Uptime: ${days}d ${hours}h ${minutes}m ${seconds}s\n⚙️ System Uptime: ${SystemUptimeForm}`;
}

function formatMemoryUsage() {
  const usedGB = Math.floor((os.totalmem() - os.freemem()) / (1024 * 1024 * 1024));
  const usedMB = Math.floor((os.totalmem() - os.freemem()) % (1024 * 1024 * 1024) / (1024 * 1024));
  const totalGB = Math.floor(os.totalmem() / (1024 * 1024 * 1024));
  const totalMB = Math.floor((os.totalmem() % (1024 * 1024 * 1024)) / (1024 * 1024));

  const usedRAM = Math.floor(os.freemem() / (1024 * 1024 * 1024));
  const usedRAMMB = Math.floor(os.freemem() % (1024 * 1024 * 1024) / (1024 * 1024));
  const totalRAM = Math.floor(os.totalmem() / (1024 * 1024 * 1024));
  const totalRAMMB = Math.floor((os.totalmem() % (1024 * 1024 * 1024)) / (1024 * 1024));

  return `Memory Usage: ${usedGB}GB ${usedMB}MB / Total ${totalGB}GB ${totalMB}MB\nRAM Usage: ${usedRAM}GB ${usedRAMMB}MB / Total ${totalRAM}GB ${totalRAMMB}MB`;
}

function formatDiskSpaceInfo(dfOutput) {
  const lines = dfOutput.split('\n');

  const diskInfo = lines[1].split(/\s+/);
  const usedDiskGB = Math.floor(parseFloat(diskInfo[2].replace('G', '')));
  const usedDiskMB = Math.floor(parseFloat(diskInfo[2].replace('G', '')) % 1 * 1024);
  const totalDiskGB = Math.floor(parseFloat(diskInfo[1].replace('G', '')));

  return `Disk Space Usage: ${usedDiskGB}GB ${usedDiskMB}MB / Total ${totalDiskGB}GB`;
}

module.exports = {
  config: {
    name: "uptime",
    version: "1.0",
    author: "Duck",
    category: "System",
    role: 0
  },
  onStart: async function ({ event, message }) {
    const cpuName = os.cpus()[0].model.split(/\s+/).slice(0, 3).join(' ');
const totalLoadPercentage = ((os.loadavg()[0] / os.cpus().length) * 100).toFixed(2);

const systemInfo = `
----------------------
⚙  System Information:
  OS: ${os.type()} ${os.release()}
  Arch: ${os.arch()}
  CPU: ${cpuName}
  Cores: ${os.cpus().length}
  Total Load Avg: ${totalLoadPercentage}%\n----------------------`;

const memoryInfo = `💾 Memory Information:\n${formatMemoryUsage()}\n----------------------`;

const dfOutput = execSync('df -h ~').toString();
const diskSpaceInfo = `📀 Disk Space Information:\n${formatDiskSpaceInfo(dfOutput)}\n----------------------`;

const processMemory = Math.round(process.memoryUsage().rss / (1024 * 1024));

SystemUptime = execSync('uptime -p').toString().trim();
const uptime = process.uptime();
const uptimeS = formatUptime(uptime, SystemUptime);

const botPing = `⚡ Bot Ping: ${Date.now() - event.timestamp} ms.`;

const fullMessage = `${botPing}\n${systemInfo}\n${memoryInfo}\n${diskSpaceInfo}\n${uptimeS}\n📊 Process Memory Usage: ${processMemory} MB\n----------------------`;

await message.reply(fullMessage);
  }
};